from enum import Enum, unique

class BaseEnum(Enum):
    @classmethod
    def values(cls) -> list:
        """Returns a list of raw values for the class"""
        values = [member.value for role, member in cls.__members__.items()]
        return values

@unique
class LogLevel(BaseEnum):
    """Represents the log levels that are supported"""

    NONE    = 0
    FAILURE = 1
    SUCCESS = 2
    ERROR   = 3
    WARN    = 4
    INFO    = 5
    DEBUG   = 6
    TRACE   = 7

@unique
class Colors(BaseEnum):
    """Represents the terminal formatting commands"""

    BLACK     = '\033[30m'
    RED       = '\033[31m'
    GREEN     = '\033[32m'
    YELLOW    = '\033[33m'
    BLUE      = '\033[34m'
    MAGENTA   = '\033[35m'
    CYAN      = '\033[36m'
    WHITE     = '\033[37m'
    RESET     = '\033[0m'
    BOLD      = '\033[1m'
    UNDERLINE = '\033[4m'
    REVERSED  = '\033[7m'


import sys
import cartils.enums as enums
import os
import colorama

FAILURE_BASE = '[FAILURE]'
SUCCESS_BASE = '[SUCCESS]'
ERROR_BASE = '[ERROR]'
WARN_BASE = '[WARN]'
INFO_BASE = '[INFO]'
DEBUG_BASE = '[DEBUG]'
TRACE_BASE = '[TRACE]'

FAILURE_PREFIX = f'{enums.Colors.BOLD.value}{enums.Colors.RED.value}{FAILURE_BASE}{enums.Colors.RESET.value} '
SUCCESS_PREFIX = f'{enums.Colors.BOLD.value}{enums.Colors.GREEN.value}{SUCCESS_BASE}{enums.Colors.RESET.value} '
ERROR_PREFIX   = f'{enums.Colors.RED.value}{ERROR_BASE}{enums.Colors.RESET.value} '
WARN_PREFIX    = f'{enums.Colors.YELLOW.value}{WARN_BASE}{enums.Colors.RESET.value} '
INFO_PREFIX    = f'{enums.Colors.BLUE.value}{INFO_BASE}{enums.Colors.RESET.value} '
DEBUG_PREFIX   = f'{enums.Colors.CYAN.value}{DEBUG_BASE}{enums.Colors.RESET.value} '
TRACE_PREFIX   = f'{enums.Colors.MAGENTA.value}{TRACE_BASE}{enums.Colors.RESET.value} '

class Logger:
    def __init__(self, log_level):
        colorama.init()
        if log_level == 'NONE':
            self.LOG_LEVEL = enums.LogLevel.NONE.value
        elif log_level == 'FAILURE':
            self.LOG_LEVEL = enums.LogLevel.SUCCESS.value
        elif log_level == 'SUCCESS':
            self.LOG_LEVEL = enums.LogLevel.SUCCESS.value
        elif log_level == 'ERROR':
            self.LOG_LEVEL = enums.LogLevel.ERROR.value
        elif log_level == 'INFO':
            self.LOG_LEVEL = enums.LogLevel.INFO.value
        elif log_level == 'WARN':
            self.LOG_LEVEL = enums.LogLevel.WARN.value
        elif log_level == 'DEBUG':
            self.LOG_LEVEL = enums.LogLevel.DEBUG.value
        elif log_level == 'TRACE':
            self.LOG_LEVEL = enums.LogLevel.TRACE.value        

    def FAILURE(self, message):
        if self.LOG_LEVEL >= enums.LogLevel.FAILURE.value:
            print('{}{}'.format(FAILURE_PREFIX, message))

    def SUCCESS(self, message):
        if self.LOG_LEVEL >= enums.LogLevel.SUCCESS.value:
            print('{}{}'.format(SUCCESS_PREFIX, message))

    def ERROR(self, message):
        if self.LOG_LEVEL >= enums.LogLevel.ERROR.value:
            print('{}{}'.format(ERROR_PREFIX, message))

    def WARN(self, message):
        if self.LOG_LEVEL >= enums.LogLevel.WARN.value:
            print('{}{}'.format(WARN_PREFIX, message))

    def INFO(self, message):
        if self.LOG_LEVEL >= enums.LogLevel.INFO.value:
            print('{}{}'.format(INFO_PREFIX, message))

    def DEBUG(self, message):
        if self.LOG_LEVEL >= enums.LogLevel.DEBUG.value:
            print('{}{}'.format(DEBUG_PREFIX, message))
    
    def TRACE(self, message):
        if self.LOG_LEVEL >= enums.LogLevel.TRACE.value:
            print('{}{}'.format(TRACE_PREFIX, message))

if __name__ == '__main__':
    logger = Logger('TRACE')
    logger.FAILURE("failure test")
    logger.SUCCESS("success test")
    logger.ERROR("error test")
    logger.WARN("warn test")
    logger.INFO("info test")
    logger.DEBUG("debug test")
    logger.TRACE("trace test")


"""
This class handles Run Length Encoding (RLE) encoding and decoding
"""
class RLE:
    """
    Encode a string with RLE
    Output is in alternating block of runs of spaces and direct copies in hex format
    """
    @staticmethod
    def encodes(data):
        arr = list(data)
        rle_mode = False

        index = 0
        rle_count = 0
        copy_buffer = ''
        out = []

        if arr[0] == ' ':
            rle_mode = True
        while index < len(arr):
            if rle_mode:
                if arr[index] == ' ':
                    rle_count += 1
                else:
                    out.append(str(rle_count))
                    rle_count = 0
                    rle_mode = False

                    hex_string = hex(ord(arr[index])).replace('0x', '')
                    if len(hex_string) < 2:
                        hex_string = '0' + hex_string
                    copy_buffer += hex_string
            else:
                if arr[index] != ' ':
                    hex_string = hex(ord(arr[index])).replace('0x', '')
                    if len(hex_string) < 2:
                        hex_string = '0' + hex_string
                    copy_buffer += hex_string
                else:
                    out.append(copy_buffer)
                    copy_buffer = ''
                    rle_mode = True
                    rle_count += 1
            index += 1
        if rle_mode:
            out.append(str(rle_count))
            out = ['1'] + out
        else:
            out.append(copy_buffer)
            out = ['0'] + out
        return '.'.join(out)

    """
    Decode a RLE string
    """
    @staticmethod
    def decodes(data):
        arr = data.split('.')
        out = ''

        if arr[0] == '1':
            rle_mode = True
        else:
            rle_mode = False

        for el in arr[1:]:
            if rle_mode:
                count = int(el)
                out += ' ' * count
            else:
                hex_chars = [el[i:i+2] for i in range(0, len(el), 2)]
                for char in hex_chars:
                    if char.startswith('0'):
                        char = char[1:]
                    out += chr(int('0x'+ char, 16))
            rle_mode = not rle_mode
        return out

    @staticmethod
    def encode(data, file_obj):
        encoded = RLE.encodes(data)
        file_obj.write(encoded)

    @staticmethod
    def decode(file_obj):
        data = file_obj.read()
        decoded = RLE.decodes(data)
        return decoded

if __name__ == '__main__':
    input_data = '     abc     abc  abc        '
    encoded = RLE.encodes(input_data)
    print(encoded)
    decoded = RLE.decodes(encoded)
    print(decoded)


import os
import subprocess

def cmd(cmd, redirect=False):
    # create a subprocess to run the command
    if redirect:
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
        # wait for it to finish and grab the output
        output = p.communicate()[0].decode()
        # grab and return the exit code
        rc = p.poll()
        return rc, output
    else:
        p = subprocess.Popen(cmd, shell=True)
        # wait for it to finish
        p.wait()
        # grab and return the exit code
        rc = p.poll()
        return rc

def check_program(program):
    if os.name == 'nt':
        if cmd(f'where {program}', redirect=True)[0] == 0:
            return True
    else:
        if cmd(f'command -v {program}', redirect=True)[0] == 0:
            return True
    return False

if __name__ == '__main__':
    print(check_program('python'))


import time
import cartils.enums as enums
import encoding
import multiprocessing

class Animation:
    def __init__(self):
        self.ANIMATION = []
        self.MAX_LINE_WIDTH = 48
        self.animation_process = None
        self.formatting = {
            '[BLACK]': enums.Colors.BLACK.value,
            '[RED]': enums.Colors.RED.value,
            '[GREEN]': enums.Colors.GREEN.value,
            '[YELLOW]': enums.Colors.YELLOW.value,
            '[BLUE]': enums.Colors.BLUE.value,
            '[MAGENTA]': enums.Colors.MAGENTA.value,
            '[CYAN]': enums.Colors.CYAN.value,
            '[WHITE]': enums.Colors.WHITE.value,
            '[RESET]': enums.Colors.RESET.value,
            '[BOLD]': enums.Colors.BOLD.value,
            '[UNDERLINE]': enums.Colors.UNDERLINE.value,
            '[REVERSED]': enums.Colors.REVERSED.value
        }

    def run(self, frame_count=-1, refresh_rate=0.25, loop=False):
        i = 0
        counter = 0
        while counter < frame_count:
            for j in range(0, len(self.ANIMATION[0])):
                line = self.ANIMATION[i % len(self.ANIMATION)][j]
                for k in self.formatting:
                    line = line.replace(k, self.formatting[k])
                print(line)
            time.sleep(refresh_rate)
            for j in range(0, len(self.ANIMATION[0])):
                print ("\033[A{}\033[A".format(' ' * self.MAX_LINE_WIDTH))
            i += 1
            i = i % len(self.ANIMATION)
            if loop:
                if counter == frame_count - 1:
                    counter = -1
                    i = 0
            counter += 1
            
    def run_async(self, frame_count=-1, refresh_rate=0.25, loop=False):
        self.animation_process = None
        self.animation_process = multiprocessing.Process(target=self.run, args=(frame_count, refresh_rate, loop))
        self.animation_process.start()
                
    def stop_async(self):
        if self.animation_process:
            self.animation_process.terminate()

    def read_animation(self, f_name, line_count, skip_line=True, rle=False):
        with open(f_name) as f:
            if rle:
                lines = encoding.RLE.decode(f).split('\n')
            else:
                lines = f.read().split('\n')
        index = 0
        data = []
        while index < len(lines):
            frame = []
            for i in range(0, line_count):
                frame.append(lines[index])
                index += 1
            data.append(frame)
            if skip_line:
                index += 1
        self.ANIMATION = data

if __name__ == '__main__':
    ff = Animation()
    ff.read_animation('credits.txt', 14)
    ff.run(frame_count=24)
    print('Now add a little bit of _spice_')
    ff.read_animation('credits.rle', 14, rle=True)
    ff.run(frame_count=24)
    print('now for some async')
    ff.run_async(frame_count=24)
    time.sleep(1)
    ff.stop_async()


import setuptools

with open("README.md", "r") as f:
    long_description = f.read()

with open("cartils/VERSION", "r") as f:
    version = f.read()

setuptools.setup(
    name="cartils",
    version=version,
    author="John Carter",
    author_email="jfcarter2358@gmail.com",
    description="A collection of useful Python utilities",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jfcarter2358/cartils",
    packages=setuptools.find_packages(),
    python_requires=">=3.7",
    install_requires=[
        'colorama==0.4.4'
    ]
)
